unit MAKCheckForDuplicatesUnit;

{******************************************************************************}
{*                             MAKPalmToss                                    *}
{*                                                                            *}
{* (C) Copyright 2003 Dmitry Malenok                                          *}
{*                                                                            *}
{* FIDO mail tosser for FIDOReader form palm created by                       *}
{*                                      Sergey Konuhov (2:5022/59.14@fidonet) *}
{* Based on sources of Palm_Tosser created by Sergey Konuhov                  *}
{*                                                                            *}
{* It was creater for following compilers:                                    *}
{*   Windows 9x/NT/2K/XP   -  Delphi 6                                        *}
{*   OS/2 Warp 3/4/4.5     -  Free Pascal 1.0.7 (build 2003/03/01 or later)   *}
{*                                                                            *}
{*                                                                            *}
{******************************************************************************}

interface

 uses
     Classes, SysUtils, PTTypes, MAKFidoLib, MAKFidoPDBLib;

 Type
  tMCFDAction                        =(mcfdDelete,mcfdMove);

  tMCFDActionRec                     =record
                                       Action      :tMCFDAction;
                                       Destination :tfAreaString;
                                      end;

  tCBSArray                          =array[0..MaxLongInt div 2] of Byte;
  pCBSArray                          =^tCBSArray;


  tCicleBufferStream                 =class(tStream)
                                       private
                                       protected
                                        FSize       :LongInt;
                                        FPosition   :LongInt;
                                        FRealSize   :LongInt;
                                        FBuffer     :pCBSArray;

                                        function CalcPosition(Value :LongInt) :LongInt;
                                        procedure SetPosition(Value :LongInt);
                                       public
                                        constructor Create(BufferSize :LongInt);
                                        destructor Destroy; override;

                                        function Read(var Buffer; Count: Longint): Longint; override;
                                        function Write(const Buffer; Count: Longint): Longint; override;
                                        function Seek(Offset: Longint; Origin: Word): Longint; override;

                                        procedure LoadFromStream(Stream: TStream);
                                        procedure SaveToStream(Stream: TStream);

                                        property RealSize :LongInt read FRealSize;
                                      end;

  tCicleMsgIDBufferStream            =class(tCicleBufferStream)
                                       private
                                        function GetCount: LongInt;
                                        function GetMsgID(Index: LongInt): UInt32;
                                       protected
                                       public
                                        constructor Create(BufferSize :LongInt);

                                        function ReadMsgID :UInt32;
                                        procedure WriteMsgID(MsgID :UInt32);

                                        property MsgID[Index :LongInt] :Uint32 read GetMsgID;
                                        property Count :LongInt read GetCount;
                                      end;

  tMAKMsgIDCheckFroDuplicates        =class
                                       private
                                        FBuffer  :tCicleMsgIDBufferStream;
                                       protected
                                       public
                                        constructor Create(BufferSize :LongInt);
                                        destructor Destroy; override;

                                        procedure Add(MsgID :Uint32);
                                        function Check(MsgID :UInt32) :boolean;
                                      end;

  tMAKFidoPDBMsgIDCheckFroDuplicates =class(tMAKMsgIDCheckFroDuplicates)
                                       private
                                       protected
                                       public
                                        function CheckFidoPDB(Packets :tPalmFidoDB;
                                                              const Action :tMCFDActionRec) :integer;

                                        procedure SaveToFile(const FileName :string);
                                        procedure LoadFromFile(const FileName :string);
                                      end;

implementation

{$IFNDEF OS2}
 uses
     RTLConsts;
{$ENDIF}


{ tCicleBufferStream }

function tCicleBufferStream.CalcPosition(Value: LongInt): LongInt;
begin
     Result:=Value;
     if FRealSize=0 then
      exit;
     while Result<0 do
      Inc(Result,FRealSize);
     while (Result>FRealSize) or (Result=FSize) do
      Dec(Result,FRealSize);
end;

constructor tCicleBufferStream.Create(BufferSize: LongInt);
begin
     inherited Create;
     FRealSize:=0;
     FSize:=BufferSize;
     FPosition:=0;
     GetMem(FBuffer,FSize);
     FillChar(FBuffer^,FSize,0);
end;

destructor tCicleBufferStream.Destroy;
begin
     FreeMem(FBuffer,FSize);
     inherited Destroy;
end;

procedure tCicleBufferStream.LoadFromStream(Stream: TStream);
 var
    Count :LongInt;
begin
     Count:=Stream.Size-Stream.Position;
     if Count<>0 then
      CopyFrom(Stream,Count);
end;

function tCicleBufferStream.Read(var Buffer; Count: LongInt): Longint;
 var
    j,n  :integer;
begin
     n:=0;
     if FPosition=FRealSize then
      FPosition:=0;

     while n<Count do
 begin
      if FSize-Position+n<Count then
       j:=FSize-Position
      else
       j:=Count-n;

      System.Move(pointer(LongInt(FBuffer)+Position)^,pointer(LongInt(@Buffer)+Count-n-j)^,j);
      SetPosition(Position+j);
      inc(n,j);
  end;
     Result:=Count;
end;

procedure tCicleBufferStream.SaveToStream(Stream: TStream);
begin
     Stream.CopyFrom(Self,FRealSize);
end;

function tCicleBufferStream.Seek(Offset: LongInt; Origin: Word): Longint;
begin
     case Origin of
soFromBeginning: FPosition:=Offset;
  soFromCurrent: inc(FPosition,Offset);
      soFromEnd: FPosition:=FRealSize+Offset;
     end;
     Result:=FPosition;
     SetPosition(FPosition);
end;

procedure tCicleBufferStream.SetPosition(Value: LongInt);
begin
     FPosition:=CalcPosition(Value);
end;

function tCicleBufferStream.Write(const Buffer; Count: LongInt): Longint;
 var
    j,n  :integer;
begin
     n:=0;

     inc(FRealSize,Count);
     if FRealSize>FSize then
      FRealSize:=FSize;

     while n<Count do
 begin
      if FSize-Position+n<Count then
       j:=FSize-Position
      else
       j:=Count-n;

      System.Move(pointer(LongInt(@Buffer)+Count-n-j)^,pointer(LongInt(FBuffer)+Position)^,j);

      SetPosition(Position+j);
      inc(n,j);
  end;
     Result:=Count;
end;

{ tCicleMsgIDBufferStream }

constructor tCicleMsgIDBufferStream.Create(BufferSize: LongInt);
 var
    i :LongInt;
begin
     i:=1 shl sizeof(Uint32);
     if BufferSize mod i>0 then
      BufferSize:=(BufferSize and not (i-1))+i;
     inherited Create(BufferSize);
end;

function tCicleMsgIDBufferStream.GetCount: LongInt;
begin
     Result:=FRealSize div sizeof(Uint32);
end;

function tCicleMsgIDBufferStream.GetMsgID(Index: LongInt): UInt32;
begin
     Index:=Index*sizeof(UInt32);
     if FPosition<FRealSize then
      Index:=Index*sizeof(UInt32)+FPosition;
     if Index>=FRealSize then
      Index:=Index mod FRealSize;
     while Index<0 do
      inc(Index,FRealSize);

     Result:=UInt32(pointer(LongInt(FBuffer)+Index)^);
end;


function tCicleMsgIDBufferStream.ReadMsgID: UInt32;
begin
     if Read(Result,sizeof(Result))<>sizeof(Result) then
      raise EStreamError.Create(SReadError);
end;

procedure tCicleMsgIDBufferStream.WriteMsgID(MsgID: UInt32);
begin
     if Write(MsgID,sizeof(MsgID))<>sizeof(MsgID) then
      raise EStreamError.Create(SWriteError);
end;

{ tMAKMsgIDCheckFroDuplicates }

procedure tMAKMsgIDCheckFroDuplicates.Add(MsgID: UInt32);
begin
     FBuffer.Write(MsgID,sizeof(MsgID));
end;

function tMAKMsgIDCheckFroDuplicates.Check(MsgID: UInt32): boolean;
 var
    i,n          :LongInt;
    TempMsgID    :LongWord;
begin
     Result:=false;
     n:=FBuffer.Count-1;
     for i:=0 to n do
 begin
      TempMsgID:=FBuffer.MsgID[i];
      if TempMsgID=MsgID then
  begin
       Result:=true;
       break;
   end;
  end;
end;

constructor tMAKMsgIDCheckFroDuplicates.Create(BufferSize: LongInt);
begin
     inherited Create;
     FBuffer:=tCicleMsgIDBufferStream.Create(BufferSize);
end;

destructor tMAKMsgIDCheckFroDuplicates.Destroy;
begin
     FBuffer.Free;
     inherited Destroy;
end;

{ tMAKFidoPDBMsgIDCheckFroDuplicates }

function tMAKFidoPDBMsgIDCheckFroDuplicates.CheckFidoPDB(
  Packets: tPalmFidoDB; const Action :tMCFDActionRec): integer;
 var
    i            :integer;
    TempHeader   :tSPacket;
begin
     Result:=0;
     i:=0;
     while i<Packets.Count do
 begin
      if Check(Packets[i].MsgID) then
  begin
       inc(Result);
       case Action.Action of
 mcfdDelete: begin
                  Packets.Delete(i);
                  continue;
              end;
   mcfdMove: begin
                  TempHeader:=Packets[i];
                  TempHeader.MsgID:=GetNewMsgID;
                  FillChar(TempHeader.Area,sizeof(TempHeader.Area),0);
                  StrPCopy(TempHeader.Area,Action.Destination);
                  Packets[i]:=TempHeader;
                  continue;
              end;
       end;
   end;
      Add(Packets[i].MsgID);
      inc(i);
  end;
end;

procedure tMAKFidoPDBMsgIDCheckFroDuplicates.LoadFromFile(
  const FileName: string);
 var
    f :TFileStream;
begin
     FBuffer.Position:=0;
     f:=TFileStream.Create(FileName,fmOpenRead+fmShareDenyWrite);
     FBuffer.LoadFromStream(f);
     f.Free;
end;

procedure tMAKFidoPDBMsgIDCheckFroDuplicates.SaveToFile(
  const FileName: string);
 var
    f :TFileStream;
begin
     FBuffer.Position:=0;
     f:=TFileStream.Create(FileName,fmCreate);
     FBuffer.SaveToStream(f);
     f.Free;
end;

end.
