unit MAKFidoLib;

{******************************************************************************}
{*                             MAKPalmToss                                    *}
{*                                                                            *}
{* (C) Copyright 2003 Dmitry Malenok                                          *}
{*                                                                            *}
{* FIDO mail tosser for FIDOReader form palm created by                       *}
{*                                      Sergey Konuhov (2:5022/59.14@fidonet) *}
{* Based on sources of Palm_Tosser created by Sergey Konuhov                  *}
{*                                                                            *}
{* It was creater for following compilers:                                    *}
{*   Windows 9x/NT/2K/XP   -  Delphi 6                                        *}
{*   OS/2 Warp 3/4/4.5     -  Free Pascal 1.0.7 (build 2003/03/01 or later)   *}
{*                                                                            *}
{*                                                                            *}
{******************************************************************************}

{$H+}

interface

 uses
{$IFDEF OS2}
     OS2Def,
{$ELSE}
     Windows,DateUtils,
{$ENDIF}
     Classes, SysUtils, PTTypes;

 const
      SInvalidStreamFormat              ='Invalid stream format.';

      FIDO_LIB_MAX_DATE_LEN             =20;
      FIDO_LIB_MAX_ADDR_LEN             =16;
      FIDO_LIB_MAX_NAME_LEN             =36;
      FIDO_LIB_MAX_SUBJ_LEN             =72;
      FIDO_LIB_MAX_AREA_LEN             =40;
      FIDO_LIB_MAX_TEXT_LEN             =(1024*32);

      FIDO_LIB_MAX_PACKETS_NUM          =High(SmallInt)+1{ 32768};
//      FIDO_LIB_MAX_BUF_LEN            =High(UShort)+1;


//* ------  p   p .MSG  ----------------------------------- */
//*     [+] - y  Pkt      */
      Msg_Private  =$0001;       //*+Pvt -                             */
      Msg_Crash    =$0002;       //*+Cra - pp                */
      Msg_Read     =$0004;       //* Rec - p y             */
      Msg_Sent     =$0008;       //* Snt -                            */
      Msg_File     =$0010;       //*+Att -                           */
      Msg_Transit  =$0020;       //* Trs - p                        */
      Msg_Orphan   =$0040;       //* Orp - y  yy          */
      Msg_Kill     =$0080;       //* K/s -                */
      Msg_Local    =$0100;       //* Loc -    y            */
      Msg_Hold     =$0200;       //* Hld -   y y       */
      Msg_FReq     =$0800;       //* FRq - p                       */
      Msg_RReq     =$1000;       //*+RRq - p p p       */
      Msg_IsRR     =$2000;       //*+RRc - p p              */
      Msg_AReq     =$4000;       //*+ARq - Audit request ???                 */
      Msg_FUpsReq  =$8000;       //* URq - p             */

 type
//* pyyp                  */
     FD_Message =packed record
                  FromUser    :array[0..FIDO_LIB_MAX_NAME_LEN-1] of Char; //*  p  From:                  */
                  ToUser      :array[0..FIDO_LIB_MAX_NAME_LEN-1] of Char; //*  y   To:                    */
                  Subject     :array[0..FIDO_LIB_MAX_SUBJ_LEN-1] of Char; //*  Subj:                              */
                  Date        :array[0..FIDO_LIB_MAX_DATE_LEN-1] of Char; //*    [dd mmm yy hh:mm:ss] */
                  TimesRead   :UInt16;                                    //* Number of times message read            */
                  DestNode    :UInt16;                                    //* Hp  y   To:             */
                  OrigNode    :UInt16;                                    //* Hp  p  From:           */
                  Cost        :UInt16;                                    //*  p               */
                  OrigNet     :UInt16;                                    //* Hp  p  From:           */
                  DestNet     :UInt16;                                    //* Hp  y   To:             */
                  DateWritten :UInt32;                                    //*           (DOS p)      */
                  DateArrived :UInt32;                                    //*  p/   (DOS p)      */
                  ReplyTo     :UInt16;                                    //*   py       */
                  Attr        :UInt16;                                    //*  py                         */
                  ReplyNext   :UInt16;                                    //*   y            */
                 end;

//* ------     p .PKT  -------------------------------- */

//*  .PKT  y y                                      */

 const
      PktVersion =2;             //* y   p 2     */
      CW_2Plus   =$0001;         //*Support for Type 2+ bundles              */
      CW_N2Plus  =$0100;         //*Validation for CW_2PLUS                  */

//* pyyp   .PKT                                                                                             */

 type
     tFIDO_PacketHeader
                  =packed record
                    OrigNode   :UInt16;               //* Hp  p  From:                 */
                    DestNode   :UInt16;               //* Hp  y   To:                           */
                    Year       :UInt16;               //*   ()                                           */
                    Month      :UInt16;               //*   ()                   */
                    Day        :UInt16;               //*   ()                    */
                    Hour       :UInt16;               //* p  ()                   */
                    Minute     :UInt16;               //* p  (y)                 */
                    Second     :UInt16;               //* p  (y)                */
                    Rate       :UInt16;               //* Baudrate = 0  ( y)         */
                    Version    :UInt16;               //* p p  = 2               */
                    OrigNet    :UInt16;               //* Hp  p  From:           */
                    DestNet    :UInt16;               //* Hp  y   To:             */
                    PCodeLo    :UInt8;                //*   pp -low            */
                    PRevMajor  :UInt8;                //* p  ppp           */
                    Password   :array [0..7] of char; //* p    ( <nul>)            */
                    QMOrigZone :UInt16;               //* = OrigZone  ( QMail)                 */
                    QMDestZone :UInt16;               //* = DestZone  (QMail)                     */
                    AuxNet     :UInt16;               //* Auxillary net = 0  ( y)    */
                    CWValidate :UInt16;               //* CapWord validation = 0x0100             */
                    PCodeHi    :UInt8;                //*   pp -high           */
                    PRevMinor  :UInt8;                //*   pp          */
                    CWCapWord  :UInt16;               //* Capabilities = 0x0001                   */
                    OrigZone   :UInt16;               //* Hp  p  From:           */
                    DestZone   :UInt16;               //* Hp  y   To:             */
                    OrigPoint  :UInt16;               //* Hp  p  From:         */
                    DestPoint  :UInt16;               //* Hp  y   To:           */
                    LongData   :UInt32;               //*   p                  */
                   end;

//*---------   y  Pkt   ---------------------------- */
     tFIDO_MsgInPktHeader
                  =packed record
                    Version   :UInt16;    //* p   = 2                      */
                    OrigNode  :UInt16;    //* Hp y p                  */
                    DestNode  :UInt16;    //* Hp  y                   */
                    OrigNet   :UInt16;    //* Hp  p                  */
                    DestNet   :UInt16;    //* Hp  y                   */
                    Attr      :UInt16;    //*                     */
                    Cost      :Uint16;    //*  p               */
                                         //*  y ASCIIz :              */
                                         //* AsciiDate - p   p  */
                                         //* ToUser    -  y  To:         */
                                         //* FromUser  -  p From:       */
                                         //* Subject   -  Subj:            */
                                         //* MsgText   -        */
                   end;

   tfDateString     =string[FIDO_LIB_MAX_DATE_LEN];
   tfUserNameString =string[FIDO_LIB_MAX_NAME_LEN];
   tfSubjectString  =string[FIDO_LIB_MAX_SUBJ_LEN];
   tfAreaString     =string[FIDO_LIB_MAX_AREA_LEN];
   tfAddressString  =string[FIDO_LIB_MAX_ADDR_LEN];
   tfText           =record  {array of char;}
                      Size  :LongInt;
                      Buf   :pchar;
                     end;

   tFIDOPktMessage  =class
                      private
                       FOriginNode      :UInt16;
                       FDestNode        :UInt16;
                       FOriginNet       :UInt16;
                       FDestNet         :UInt16;
                       FAttr            :UInt16;
                       FCost            :Uint16;
                       FMsgID           :UInt32;
                       FReply           :UInt32;
                       FDate            :tDateTime;
                       FFromUser        :tfUserNameString;
                       FFromAddr        :tfAddressString;
                       FToUser          :tfUserNameString;
                       FSubject         :tfSubjectString;
                       FArea            :tfAreaString;
                       FText            :tStringList;
                       FKludges         :tStringList;
                       FSeenBy          :tStringList;
                      protected
                       function HexToInt(const s :string) :UInt32;
//                       function OEMToAnsiStr(const s:string) :string;
                       function FindZeroInTheStream(Stream :tStream) :boolean;
                       function LoadStreamToZero(Stream :tStream) :tMemoryStream;
                      public
                       constructor Create;
                       destructor Destroy; override;

                       procedure LoadFromStream(Stream :tStream); virtual;

                       procedure SetData(const AHeader :tFIDO_MsgInPktHeader;
                                         const AAsciiDate :tfDateString;
                                         const AToName,AFromName :tfUserNameString;
                                         const ASubject :tfSubjectString;
                                         const AText :tfText);

                       property OriginNode      :UInt16 read FOriginNode write FOriginNode;
                       property DestNode        :UInt16 read FDestNode write FDestNode;
                       property OriginNet       :UInt16 read FOriginNet write FOriginNet;
                       property DestNet         :UInt16 read FDestNet write FDestNet;
                       property Attr            :UInt16 read FAttr write FAttr;
                       property Cost            :Uint16 read FCost write FCost;
                       property MsgID           :UInt32 read FMsgID write FMsgID;
                       property Reply           :UInt32 read FReply write FReply;
                       property Date            :tDateTime read FDate write FDate;
                       property FromUser        :tfUserNameString read FFromUser write FFromUser;
                       property FromAddr        :tfAddressString read FFromAddr write FFromAddr;
                       property ToUser          :tfUserNameString read FToUser write FToUser;
                       property Subject         :tfSubjectString read FSubject write FSubject;
                       property Area            :tfAreaString read FArea write FArea;
                       property Text            :tStringList read FText;
                       property Kludges         :TStringList read FKludges;
                       property SeenBy          :TStringList read FSeenBy;
                     end;

   tFIDOPktMessageList
                    =class(tList)
                      private
                      protected
                       function GetItem(Index: LongInt): tFIDOPktMessage; virtual;
                       procedure SetItem(Index: LongInt; const Value: tFIDOPktMessage); virtual;
                      public
                       function MakeNew :tFIDOPktMessage; virtual;
                       procedure Clear; override;
                       procedure Delete(Index :LongInt); virtual;

                       procedure SortPerArea;

                       property Items[Index :LongInt] :tFIDOPktMessage read GetItem write SetItem; default;
                     end;

     tMAKFidoPacket
                 =class
                   private
                    FHeader       :tFIDO_PacketHeader;
                    FPackets      :tFIDOPktMessageList;
                   protected
                   public
                    constructor Create;
                    destructor Destroy; override;

                    procedure LoadFromStream(Stream :tStream); virtual;

                    procedure LoadFromFile(const FileName :string); virtual;

                    property Header :tFIDO_PacketHeader read FHeader write FHeader;
                    property Packets :tFIDOPktMessageList read FPackets;
                  end;

 function GetNewMsgID :Uint32;

implementation

{$IFDEF OS2}
 uses
     DosCalls;
{$ENDIF}

 var
    LastMsgID  :UInt32;

function GetNewMsgID :Uint32;
 begin
      Result:=LastMsgID;
      inc(LastMsgID);
  end;

{ tFIDOPktMessage }
constructor tFIDOPktMessage.Create;
begin
     inherited Create;

     FText:=TStringList.Create;
     FKludges:=TStringList.Create;
     FSeenBy:=TStringList.Create;
end;

destructor tFIDOPktMessage.Destroy;
begin
     FSeenBy.Free;
     FKludges.Free;
     FText.Free;

     inherited Destroy;
end;


function tFIDOPktMessage.HexToInt(const s: string): UInt32;
 var
    i :LongInt;
    b :byte;
begin
     Result:=0;
     for i:=1 to length(s) do
      if LowerCase(s)[i] in ['0'..'9','a'..'f'] then
  begin
       b:=ord(LowerCase(s)[i])-$30;
       if b>9 then
        b:=b-7;
       Result:=(Result shl 4) or (b and $f);
   end
      else
       break;
end;


function tFIDOPktMessage.FindZeroInTheStream(Stream :tStream) :boolean;
 type
     tBufType   =array[0..4095] of Byte;
     pBufType   =^tBufType;
 var
    ReadBuffer      :pBufType;
    i,n             :longint;
    b               :byte;
begin
     b:=1;
     GetMem(ReadBuffer,sizeof(tBufType));

     while Stream.Position<=Stream.Size do
 begin
      b:=1;
      n:=Stream.Read(ReadBuffer^,sizeof(tBufType));
      for i:=0 to n-1 do
  begin
       b:=ReadBuffer[i];
       if b=0 then
   begin
        Stream.Position:=Stream.Position-n+i+1;
        break;
    end;
   end;
      if b=0 then break;
  end;

     FreeMem(ReadBuffer,sizeof(tBufType));
     Result:=b=0;
end;

procedure tFIDOPktMessage.LoadFromStream(Stream: tStream);

 function ReadAsciizString :string;
   var
      Size            :LongInt;
      TempStream      :tMemoryStream;
  begin
       TempStream:=LoadStreamToZero(Stream);
       if not Assigned(TempStream) then
        raise EStreamError.Create(SInvalidStreamFormat);

       Result:='';
       Size:=TempStream.Position;
       TempStream.Position:=0;
       if Size>0 then
   begin
        SetLength(Result,Size);
        TempStream.Read(Result[1],Size);
    end;
       TempStream.Free;
   end;

 var
    TempHeader      :tFIDO_MsgInPktHeader;
    TempText        :tfText;
    TempDate        :tfDateString;
    TempToName,
    TempFromName    :tfUserNameString;
    TempSubjectName :tfSubjectString;
    TempStream      :tMemoryStream;

begin
     Stream.Read(TempHeader,sizeof(TempHeader));
     if TempHeader.Version<>2 then
      raise EStreamError.Create(SInvalidStreamFormat);

     TempDate:=OEMToAnsiStr(ReadAsciizString);
     TempToName:=OEMToAnsiStr(ReadAsciizString);
     TempFromName:=OEMToAnsiStr(ReadAsciizString);
     TempSubjectName:=OEMToAnsiStr(ReadAsciizString);

     TempStream:=LoadStreamToZero(Stream);
     if not Assigned(TempStream) or (TempStream.Position=0) then
 begin
      TempStream.Free;
      raise EStreamError.Create(SInvalidStreamFormat);
      exit;
  end;

     TempText.Size:=TempStream.Position;
     TempStream.Position:=0;
     GetMem(TempText.Buf,TempText.Size);
     TempStream.Read(TempText.Buf^,TempText.Size);

{$IFDEF OS2}
      AltToWin(TempText.Buf,TempText.Buf,TempText.Size);
{$ELSE}
{$IFDEF WIN32}
      OemToCharBuff(TempText.Buf, TempText.Buf, TempText.Size);
{$ELSE}
      OemToAnsiBuff(@TempText[0], @TempText[0], High(TempText)+1);
{$ENDIF}
{$ENDIF}

     SetData(TempHeader,TempDate,TempToName,TempFromName,TempSubjectName,
             TempText);
     FreeMem(TempText.Buf,TempText.Size);
     TempStream.Free;
end;

function tFIDOPktMessage.LoadStreamToZero(Stream :tStream) :tMemoryStream;
 const
      LoadCount =4096;
 var
    i,j,n           :longint;
    b               :byte;
begin
     Result:=tMemoryStream.Create;
     j:=0;

     while Stream.Position<=Stream.Size do
 begin
      if Stream.Position+LoadCount<Stream.Size then
       n:=Result.CopyFrom(Stream,LoadCount)
      else
       n:=Result.CopyFrom(Stream,Stream.Size-Stream.Position);
      for i:=j to j+n-1 do
  begin
       b:=pByteArray(Result.Memory)[i];
       if b=0 then
   begin
        Result.Position:=i;
        Stream.Position:=Stream.Position-n+(i-j)+1;
        exit;
    end;
   end;
      Inc(j,n);
  end;

     Result.Free;
     Result:=nil;
end;


procedure tFIDOPktMessage.SetData(const AHeader: tFIDO_MsgInPktHeader;
                                  const AAsciiDate :tfDateString;
                                  const AToName,AFromName :tfUserNameString;
                                  const ASubject :tfSubjectString;
                                  const AText :tfText);
 var
    i,j  :LongInt;
    s    :AnsiString;
begin
     FOriginNode:=AHeader.OrigNode;
     FDestNode:=AHeader.DestNode;
     FOriginNet:=AHeader.OrigNet;
     FDestNet:=AHeader.DestNet;
     FAttr:=AHeader.Attr;
     FCost:=AHeader.Cost;
     FFromUser:=AFromName;
     FToUser:=AToName;
     FSubject:=ASubject;
     FDate:=AsciiDateToDateTime(AAsciiDate);

     FArea:='';
     FFromAddr:='';
     Reply:=0;
     MsgID:=0;

     s:='';
     FText.Clear;
     FKludges.Clear;
     FSeenBy.Clear;

     for i:=0 to AText.Size-1 do
 begin
      case AText.Buf[i] of
   #$0d: begin
              if (length(s)>0) and (s[1]=#1) then
          begin
               s:=copy(s,2,length(s)-1);
               if copy(s,1,6)='MSGID:' then
                FMsgID:=HexToInt(copy(s,length(s)-7,8))
               else
                if copy(s,1,6)='REPLY:' then
                 FReply:=HexToInt(copy(s,length(s)-7,8))
                else
                 FKludges.Add(s);
           end
              else
          begin
               if (FArea='') and (copy(s,1,5)='AREA:') then
                FArea:=UpperCase(copy(s,6,length(s)-5))
               else
                if copy(s,1,9)='SEEN-BY: ' then
                 FSeenBy.Add(copy(s,10,length(s)-9))
                else
                 if copy(s,1,10)=' * Origin:' then
             begin
                  FText.Add(s);

                  j:=length(s);
                  while (j>0) and (s[j]<>'(') do
                   dec(j);
                  if j>0 then
              begin
                   inc(j);
                   FFromAddr:='';
                   while (j<=length(s)) and not (s[j] in [')','@',#0,#$0d]) do
               begin
                    FFromAddr:=FFromAddr+s[j];
                    inc(j);
                end;
               end;
              end
                 else
                  FText.Add(s);
           end;
              s:='';
          end;
       else
        s:=s+AText.Buf[i];
      end;
  end;
end;

{ tFIDOPktMessageList }
function CompArea(Item1, Item2: Pointer): LongInt;
begin
      Result:=1;
      if tFIDOPktMessage(Item1).Area>tFIDOPktMessage(Item2).Area then
       Result:=2;
      if tFIDOPktMessage(Item1).Area=tFIDOPktMessage(Item2).Area then
       Result:=0;
end;

procedure tFIDOPktMessageList.Clear;
 var
    i :LongInt;
begin
     for i:=0 to Count-1 do
      if Assigned(Items[i]) then
       Items[i].Free;
     inherited Clear;
end;

procedure tFIDOPktMessageList.Delete(Index: LongInt);
begin
     if Assigned(Items[Index]) then
      Items[Index].Free;
     inherited Delete(Index);
end;

function tFIDOPktMessageList.GetItem(Index: LongInt): tFIDOPktMessage;
begin
     Result:=tFIDOPktMessage(inherited Items[Index]);
end;

function tFIDOPktMessageList.MakeNew: tFIDOPktMessage;
begin
     Result:=tFIDOPktMessage.Create;
     Add(Result);
end;

procedure tFIDOPktMessageList.SetItem(Index: LongInt;
  const Value: tFIDOPktMessage);
begin
     if Assigned(Items[Index]) then
      Items[Index].Free;
     inherited Items[Index]:=Value;
end;

procedure tFIDOPktMessageList.SortPerArea;
begin
     Sort(CompArea);
end;

{ tMAKFidoPacket }

constructor tMAKFidoPacket.Create;
begin
     inherited Create;

     FPackets:=tFIDOPktMessageList.Create;
end;

destructor tMAKFidoPacket.Destroy;
begin
     FPackets.Clear;
     FPackets.Free;
     inherited Destroy;
end;

procedure tMAKFidoPacket.LoadFromFile(const FileName: string);
 var
    f :tFileStream;
begin
     f:=TFileStream.Create(FileName,fmOpenRead+fmShareDenyWrite);
     LoadFromStream(f);
     f.Free;
end;

procedure tMAKFidoPacket.LoadFromStream(Stream: tStream);
 var
    w          :word;
    TempPacket :tFIDOPktMessage;
begin
     Stream.Read(FHeader,sizeof(FHeader));
     while Stream.Position+sizeof(tFIDO_MsgInPktHeader)+1<=Stream.Size do
 begin
      TempPacket:=Packets.MakeNew;
      TempPacket.LoadFromStream(Stream);
  end;

     Stream.Read(w,sizeof(w));
     if (w<>0) or (Stream.Position<>Stream.Size) then
      raise EStreamError.Create(SInvalidStreamFormat);
end;

{$IFDEF OS2}
var
   k :LongWord;
{$ENDIF}
initialization
{$IFDEF OS2}
     if DosQuerySysInfo(svMSCount,svMSCount,k,sizeof(k))<>No_Error then
 begin
      Randomize;
      k:=Random($100);
  end;
     LastMsgID:=(DateTimeToUnix(Now) and $ffffff) or
                 ((k and $ff) shl $18);

{$ELSE}
     LastMsgID:=DateTimeToUnix(Now) and $ffffff+
            (GetTickCount and $ff) shl $18;
{$ENDIF}

end.
