unit MAKFidoPDBLib;

{******************************************************************************}
{*                             MAKPalmToss                                    *}
{*                                                                            *}
{* (C) Copyright 2003 Dmitry Malenok                                          *}
{*                                                                            *}
{* FIDO mail tosser for FIDOReader form palm created by                       *}
{*                                      Sergey Konuhov (2:5022/59.14@fidonet) *}
{* Based on sources of Palm_Tosser created by Sergey Konuhov                  *}
{*                                                                            *}
{* It was creater for following compilers:                                    *}
{*   Windows 9x/NT/2K/XP   -  Delphi 6                                        *}
{*   OS/2 Warp 3/4/4.5     -  Free Pascal 1.0.7 (build 2003/03/01 or later)   *}
{*                                                                            *}
{*                                                                            *}
{******************************************************************************}

{$H+}

interface

 uses
{$IFNDEF OS2}
     DateUtils, 
{$ENDIF}
     SysUtils,PTTypes, MAKPDBLib, MAKFidoLib;

 type
     tSPacket    =record
                   MsgID,
                   Reply      :UInt32;
                   Area       :array [0..FIDO_LIB_MAX_AREA_LEN-1] of char;
                   Date       :array [0..FIDO_LIB_MAX_DATE_LEN-1] of char;
                   From       :array [0..FIDO_LIB_MAX_NAME_LEN-1] of char;
                   Addr       :array [0..FIDO_LIB_MAX_ADDR_LEN-1] of char;
                   NTo        :array [0..FIDO_LIB_MAX_NAME_LEN-1] of char;
                   Subj       :array [0..FIDO_LIB_MAX_SUBJ_LEN-1] of char;
                   Attr       :UInt16;
                   Text_len   :UInt16;
                  end;
     pSPacket    =^tSPacket;

     tPalmFidoDB          =class(tPalmDB)
                            private
                             function GetItem(Index: LongWord): tSPacket;
                             function GetText(Index: LongWord): pChar;
                             procedure SetItem(Index: LongWord; const Value: tSPacket);
                             function GetTextLength(Index: LongWord): UInt16;
                            protected
                            public
                             function Add(const Value :tFIDOPktMessage) :integer;

                             function CreateCopy(Index :integer; const Header :tSPacket) :integer;

                             function SaveToDir(const FileDir,StartFileName :string; Limit :LongWord) :string; virtual;

                             procedure SortAreas;

                             property Items[Index :LongWord] :tSPacket read GetItem write SetItem; default;
                             property Text[Index :LongWord] :pChar read GetText;
                             property TextLength[Index :LongWord] :UInt16 read GetTextLength;
                           end;

implementation

 uses
     Classes;

{ tPalmFidoDB }

function CompAreas(Item1,Item2 :pointer) :longint;
 var
    d1,d2 :tDateTime;
begin
     Result:=StrComp(pSPacket(ppdbRecord(Item1)^.Buf)^.Area,pSPacket(ppdbRecord(Item2)^.Buf)^.Area);

     if Result=0 then
      try
       d1:=AsciiDateToDateTime(StrPas(pSPacket(ppdbRecord(Item1)^.Buf)^.Date));
       d2:=AsciiDateToDateTime(StrPas(pSPacket(ppdbRecord(Item2)^.Buf)^.Date));
       if d1>d2 then
        Result:=1
       else
        if d2>d1 then
         Result:=-1; 
      except
      end;
end;

function tPalmFidoDB.Add(const Value: tFIDOPktMessage): integer;
 var
    TempPacket  :tSPacket;
    TempElement :tpdbRecord;
    Buf         :pchar;
    i           :LongInt;
    s           :AnsiString;
    n           :LongInt;
    d,m,y       :word;
begin
     FillChar(TempPacket,sizeof(TempPacket),0);
     with TempPacket do
 begin
      MsgID:=Value.MsgID;
      Reply:=Value.Reply;
      Attr:=Value.Attr;
      StrPCopy(Area,Value.Area);
      DecodeDate(Value.Date,y,m,d);
      y:=y mod 100;
      StrPCopy(Date,Format('%.2d %s %.2d',[d,EngMonthNames[m],y])+FormatDateTime('  hh:nn:ss',Value.Date));
      StrPCopy(From,Value.FromUser);
      StrPCopy(Addr,Value.FromAddr);
      StrPCopy(NTo,Value.ToUser);
      StrPCopy(Subj,Value.Subject);
      s:='';
      n:=Value.Text.Count-1; {because of this fucking Sibyl}
      for i:=0 to {Value.Text.Count-1}n do
  begin
       if i>0 then
        s:=s+#$0a;
       s:=s+Value.Text[i];
   end;
      Text_len:=length(s)+1;
  end;
     TempElement.Size:=sizeof(TempPacket)+TempPacket.Text_len;
     TempPacket.Text_len:=pdbRevert16(TempPacket.Text_len);
     GetMem(Buf,TempElement.Size);
     System.Move(TempPacket,Buf^,SizeOf(TempPacket));
     StrPCopy(@Buf[sizeof(TempPacket)],s);
     TempElement.Buf:=Buf;
     Result:=inherited Add(TempElement);
end;

function tPalmFidoDB.CreateCopy(Index: integer; const Header: tSPacket) :integer;
 var
    TempPacket  :tSPacket;
    TempElement :tpdbRecord;
    Buf         :pChar;
begin
     System.Move(Header,TempPacket,sizeof(Header));
     TempElement.Size:=sizeof(TempPacket)+TextLength[Index];
     TempPacket.Text_len:=pdbRevert16(TextLength[Index]);
     GetMem(Buf,TempElement.Size);
     System.Move(TempPacket,Buf^,SizeOf(TempPacket));
     System.Move(Text[Index]^,Buf[sizeof(TempPacket)],TextLength[Index]);
     TempElement.Buf:=Buf;
     Result:=inherited Add(TempElement);
end;

function tPalmFidoDB.GetItem(Index: LongWord): tSPacket;
 var
    TempRecord :tpdbRecord;
begin
     TempRecord:=inherited Items[Index];
     Result:=pSPacket(TempRecord.Buf)^;
end;

function tPalmFidoDB.GetText(Index: LongWord): pChar;
 var
    TempRecord :tpdbRecord;
begin
     TempRecord:=inherited Items[Index];
     Result:=pChar(pChar(TempRecord.Buf)+sizeof(tSPacket));
end;

function tPalmFidoDB.GetTextLength(Index: LongWord): UInt16;
begin
     Result:=pdbRevert16(Items[Index].Text_len);
end;

function tPalmFidoDB.SaveToDir(const FileDir, StartFileName: string; Limit :LongWord) :string;
 var
    f              :tFileStream;
    i,j            :LongWord;
    s              :string;
    TempHeader     :tDatabaseHdr;
    FileName       :string;
begin
     Result:='';
     if Count=0 then exit;

     j:=DateTimeToUnix(Now);

     TempHeader:=Header;
     TempHeader.Attributes:=pdbRevert16(8);
     TempHeader.Version:=pdbRevert16(1);
     TempHeader.HType:=$6f646966;    //fido
     TempHeader.Creator:=$73556147;  //GaUs

     i:=0;
     FileName:=StartFileName;

     repeat
{$IFDEF FPC}
      s:='fido_'+IntToHex(j,8)+'_base';
{$ELSE}
      s:='fido_'+IntToHex(j,0)+'_base';
{$ENDIF}
      StrPLCopy(TempHeader.Name,s,High(TempHeader.Name)+1);
      Header:=TempHeader;
      if length(FileName)=0 then
{$IFDEF FPC}
       FileName:='fido'+IntToHex(j,8)+'.pdb';
{$ELSE}
       FileName:='fido'+IntToHex(j,0)+'.pdb';
{$ENDIF}

      f:=TFileStream.Create(FileDir+FileName,fmCreate or fmShareDenyWrite);
      i:=SaveToStream(f,i,Limit);
      f.Free;

      inc(j);
      Result:=Result+FileName+'; ';
      FileName:='';
     until i=Count;

     if length(Result)>0 then
      Result:=copy(Result,1,length(Result)-1);
end;

procedure tPalmFidoDB.SetItem(Index: LongWord; const Value: tSPacket);
 var
    TempRecord :tpdbRecord;
begin
     TempRecord:=inherited Items[Index];
     System.Move(Value,TempRecord.Buf^,SizeOf(Value));
end;

procedure tPalmFidoDB.SortAreas;
begin
     Sort(CompAreas);
end;

end.
