unit MAKFidoSearcherUnit;

{******************************************************************************}
{*                             MAKPalmToss                                    *}
{*                                                                            *}
{* (C) Copyright 2003 Dmitry Malenok                                          *}
{*                                                                            *}
{* FIDO mail tosser for FIDOReader form palm created by                       *}
{*                                      Sergey Konuhov (2:5022/59.14@fidonet) *}
{* Based on sources of Palm_Tosser created by Sergey Konuhov                  *}
{*                                                                            *}
{* It was creater for following compilers:                                    *}
{*   Windows 9x/NT/2K/XP   -  Delphi 6                                        *}
{*   OS/2 Warp 3/4/4.5     -  Free Pascal 1.0.7 (build 2003/03/01 or later)   *}
{*                                                                            *}
{*                                                                            *}
{******************************************************************************}

{$H+}

interface

 uses
     Classes, SysUtils, PTTypes, MAKFidoLib, MAKFidoPDBLib, MAKSearcherUnit;

 type
     tMFSMaskElement         =record
                               FromName  :tMAKStringSearcher;
                               FromAddr  :tMAKStringSearcher;
                               ToName    :tMAKStringSearcher;
                               Subject   :tMAKStringSearcher;
                              end;
     pMFSMaskElement         =^tMFSMaskElement;

     tMFSActionCommand       =(mfsaNone,mfsaCopy,mfsaMove,mfsaDelete);

     tMFSAction              =record
                               case Command :tMFSActionCommand of
                       mfsaCopy,
                       mfsaMove: (Destination :tfAreaString);
                              end;

     pMFSAction              =^tMFSAction;

     tMFSMaskList            =class(tMAKList)
                               private
                                function GetItem(Index: integer): tMFSMaskElement;
                               protected
                                procedure ClearElement(Index :integer); override;
                               public
                                function Add(const MaskStr :string) :integer;

                                function CheckFidoPDB(const PHeader :tSPacket) :boolean;

                                property Items[Index :integer] :tMFSMaskElement
                                                          read GetItem; default;
                              end;

     tMFSActionList          =class(tMAKList)
                               private
                                function GetItem(Index: integer): tMFSAction;
                               protected
                                procedure ClearElement(Index :integer); override;
                               public
                                function Add(const ActionStr :string) :integer;

                                property Items[Index :integer] :tMFSAction
                                                          read GetItem; default;
                              end;

     tMFSElementAddResult    =(mfsearNone,mfsearMask,mfsearAction);
     tMAKFidoSearcherElement =class
                               private
                                FMasks       :tMFSMaskList;
                                FActions     :tMFSActionList;
                               protected
                               public
                                constructor Create;
                                destructor Destroy; override;

                                function Add(const s :string; OnlyActionFlag :boolean) :tMFSElementAddResult;

                                function ProcessPDB(Packet :tPalmFidoDB; Index :integer) :tMFSActionCommand;

                                property Actions :tMFSActionList read FActions;
                                property Masks   :tMFSMaskList read FMasks;
                              end;

     tMAKFidoSearcherList    =class(tMAKList)
                               private
                                FArea   :tfAreaString;
                                function GetItem(Index: integer): tMAKFidoSearcherElement;
                               protected
                                procedure ClearElement(Index :integer); override;
                               public
                                constructor Create(const AArea :string);

                                procedure Add(const Strings :tStrings; var Index :integer);

                                function ProcessPDB(const Packet :tPalmFidoDB; var Index :integer) :boolean;

                                property Items[Index :integer] :tMAKFidoSearcherElement
                                                          read GetItem; default;
                                property Area :tfAreaString read FArea;
                              end;

     tMAKFidoSearcher        =class(tMAKList)
                               private
                                function GetItem(Index: integer): tMAKFidoSearcherList;
                               protected
                                procedure ClearElement(Index :integer); override;
                                function IndexOfArea(const AArea :string) :integer; 
                               public
                                procedure Add(const Strings :tStrings);

                                procedure ProcessPDB(const Packet :tPalmFidoDB);

                                property Items[Index :integer] :tMAKFidoSearcherList
                                                          read GetItem; default;
                              end;

implementation

 const
      MFSFunctionsStr       :array [0..2] of string=('copy','move','delete');
      MFSFunctions          :array [0..2] of tMFSActionCommand=
                                                 (mfsaCopy,mfsaMove,mfsaDelete);
      MFSFunctionsFlag      :array [0..2] of boolean=
                                                 (false,false,true);
      MFSCommandID_Area     =0;
      MFSCommandID_Mask     =1;
      MFSCommandID_Action   =2;
      MFSCommandsStr        :array [0..2] of string=('area','mask','action');

      MFSAllAreasID         ='$ALL$';

function CompareCommand(const InStr,Command :string; AloneFlag :boolean) :boolean;
 var
    n,m :integer;
begin
     n:=length(Command);
     m:=Length(InStr);
     Result:=(LowerCase(copy(InStr,1,n))=LowerCase(Command)) and
             ((AloneFlag and (n=m)) or ((n<m) and (InStr[n+1] in [' ',#9])));
end;

function CompareCommandWithColon(const InStr,Command :string) :boolean;
 var
    n,m :integer;
begin
     n:=length(Command);
     m:=Length(InStr);
     Result:=(m>n) and (InStr[n+1]=':') and
             (LowerCase(copy(InStr,1,n))=LowerCase(Command));
end;

{------------------------------------------------------------------------------}
{ tMFSMaskList }

function tMFSMaskList.Add(const MaskStr: string): integer;
 var
    i,j,k,n           :integer;
    TempElement       :pMFSMaskElement;
    SlashFlag         :boolean;
    SquareBracketFlag :boolean;
    s                 :string;
begin
     new(TempElement);
     TempElement^.FromName:=tMAKStringSearcher.Create;
     TempElement^.ToName:=tMAKStringSearcher.Create;
     TempElement^.FromAddr:=tMAKStringSearcher.Create;
     TempElement^.Subject:=tMAKStringSearcher.Create;

     j:=1;
     k:=0;
     i:=1;
     n:=length(MaskStr);

     SlashFlag:=false;
     SquareBracketFlag:=false;

     while i<=n+1 do
 begin
      if (i=n+1) or ((not SlashFlag) and (not SquareBracketFlag) and
                     (MaskStr[i]=',')) then
  begin
       s:=copy(MaskStr,j,i-j);
       case k of
      0: TempElement^.FromName.SearchString:=s;
      1: TempElement^.FromAddr.SearchString:=s;
      2: TempElement^.ToName.SearchString:=s;
      3: TempElement^.Subject.SearchString:=s;
       end;

       inc(k);
       j:=i+1;
   end
      else
  begin
       SlashFlag:=(MaskStr[i]='\') and not SlashFlag;
{       SquareBracketFlag:=(SquareBracketFlag or (MaskStr[i]='[')) and
                           not (MaskStr[i]=']');}
       SquareBracketFlag:=SquareBracketFlag or (MaskStr[i]='[');
       SquareBracketFlag:=SquareBracketFlag and not (MaskStr[i]=']');
   end;

      inc(i);
  end;
     Result:=inherited Add(TempElement);
end;

function tMFSMaskList.CheckFidoPDB(const PHeader: tSPacket): boolean;
 var
    i             :integer;
    TempFromName  :string;
    TempToName    :string;
    TempFromAddr  :string;
    TempSubject   :string;
begin
     Result:=false;
     
     TempFromName:=StrPas(PHeader.From);
     TempToName:=StrPas(PHeader.NTo);
     TempFromAddr:=StrPas(PHeader.Addr);
     TempSubject:=StrPas(PHeader.Subj);

     for i:=0 to Count-1 do
 begin
      Result:=Items[i].FromName.SearchInString(TempFromName) and
              Items[i].FromAddr.SearchInString(TempFromAddr) and
              Items[i].ToName.SearchInString(TempToName) and
              Items[i].Subject.SearchInString(TempSubject);
      if Result then exit;
  end;
end;

procedure tMFSMaskList.ClearElement(Index: integer);
 var
    TempElement :pMFSMaskElement;
begin
     if Assigned(inherited Items[Index]) then
 begin
      TempElement:=inherited Items[Index];
{      pMFSMaskElement(inherited Items[Index])^.FromName.Free;
      pMFSMaskElement(inherited Items[Index])^.ToName.Free;
      pMFSMaskElement(inherited Items[Index])^.FromAddr.Free;
      pMFSMaskElement(inherited Items[Index])^.Subject.Free;
      Dispose(pMFSMaskElement(inherited Items[Index]));}
      TempElement^.FromName.Free;
      TempElement^.ToName.Free;
      TempElement^.FromAddr.Free;
      TempElement^.Subject.Free;
      Dispose(TempElement);
      inherited Items[Index]:=nil;
  end;
end;

function tMFSMaskList.GetItem(Index: integer): tMFSMaskElement;
begin
     Result:=pMFSMaskElement(inherited Items[Index])^;
end;

{------------------------------------------------------------------------------}
{ tMFSActionList }

function tMFSActionList.Add(const ActionStr: string): integer;
 var
    TempElement :pMFSAction;
    n           :integer;
    i           :integer;
begin
     new(TempElement);
     TempElement^.Command:=mfsaNone;


     for i:=0 to High(MFSFunctionsStr) do
      if CompareCommand(ActionStr,MFSFunctionsStr[i],MFSFunctionsFlag[i]) then
  begin
       TempElement^.Command:=MFSFunctions[i];
       if i<2 then
   begin
        n:=length(MFSFunctionsStr[i])+1;
        TempElement^.Destination:=Trim(copy(ActionStr,n+1,length(ActionStr)-n));
    end;
   end;
{     else
      if CompareCommand(ActionStr,'move',false) then
  begin
       n:=length('move')+1;
       TempElement.Command:=mfsaMove;
       TempElement.Destination:=Trim(copy(ActionStr,n+1,length(ActionStr)-n));
   end
      else
       if CompareCommand(ActionStr,'delete',true) then
        TempElement.Command:=mfsaDelete;}

     if TempElement^.Command<>mfsaNone then
      Result:=inherited Add(TempElement)
     else
 begin
      Result:=-1;
      Dispose(TempElement);
  end;
end;

procedure tMFSActionList.ClearElement(Index: integer);
 var
    TempAction :pMFSAction;
begin
     if Assigned(inherited Items[Index]) then
 begin
      TempAction:=pMFSAction(inherited Items[Index]);
      Dispose(TempAction);
      inherited Items[Index]:=nil;
  end;
end;

function tMFSActionList.GetItem(Index: integer): tMFSAction;
begin
     Result:=pMFSAction(inherited Items[Index])^;
end;

{------------------------------------------------------------------------------}
{ tMAKFidoSearcherElement }

function tMAKFidoSearcherElement.Add(const s: string;
                                 OnlyActionFlag :boolean): tMFSElementAddResult;
 var
    n :integer;
begin
     Result:=mfsearNone;

     if CompareCommandWithColon(s,MFSCommandsStr[MFSCommandID_Mask]) then
 begin
      if OnlyActionFlag then
  begin
       Result:=mfsearMask;
       exit;
   end;

      n:=length(MFSCommandsStr[MFSCommandID_Mask])+1;
      if Masks.Add(copy(s,n+1,length(s)-n))<>-1 then
       Result:=mfsearMask;
  end
     else
      if CompareCommandWithColon(s,MFSCommandsStr[MFSCommandID_Action]) then
  begin
       n:=length(MFSCommandsStr[MFSCommandID_Action])+1;
       if Actions.Add(trim(copy(s,n+1,length(s)-n)))<>-1 then;
        Result:=mfsearAction;
   end;
end;

constructor tMAKFidoSearcherElement.Create;
begin
     inherited Create;

     FMasks:=tMFSMaskList.Create;
     FActions:=tMFSActionList.Create;
end;

destructor tMAKFidoSearcherElement.Destroy;
begin
     FActions.Free;
     FMasks.Free;

     inherited Destroy;
end;

function tMAKFidoSearcherElement.ProcessPDB(Packet: tPalmFidoDB;
  Index: integer): tMFSActionCommand;
 var
    i            :integer;
    TempHeader   :tSPacket;
begin
     TempHeader:=Packet[Index];
     Result:=mfsaNone;
     if not Masks.CheckFidoPDB(TempHeader) then
      exit;

     for i:=0 to Actions.Count-1 do
 begin
      Result:=Actions[i].Command;
      case Result of
  mfsaCopy: begin
                 StrPCopy(TempHeader.Area,Actions[i].Destination);
                 TempHeader.MsgID:=GetNewMsgID;
                 Packet.CreateCopy(Index,TempHeader);
             end;
  mfsaMove: begin
                 FillChar(TempHeader.Area,sizeof(TempHeader.Area),0);
                 StrPCopy(TempHeader.Area,Actions[i].Destination);
                 Packet[Index]:=TempHeader;
                 exit;
             end;
mfsaDelete: begin
                 Packet.Delete(Index);
                 exit;
             end;
      end;
  end;
end;

{------------------------------------------------------------------------------}
{ tMAKFidoSearcherList }

procedure tMAKFidoSearcherList.Add(const Strings: tStrings;
  var Index: integer);
 var
    i           :integer;
    MaskFlag    :boolean;
    s           :string;
    TempElement :tMAKFidoSearcherElement;
begin
     MaskFlag:=true;
     TempElement:=tMAKFidoSearcherElement.Create;

     i:=Index;
     while i<Strings.Count do
 begin
      s:=Strings[i];
      if not((length(s)=0) or (s[1]=';')) then
  begin
       if (s[1]='[') or CompareCommandWithColon(s,MFSCommandsStr[MFSCommandID_Area]) then
        break;

       case TempElement.Add(s,not MaskFlag) of
  mfsearMask:begin
                  if not MaskFlag then
              begin
                   MaskFlag:=true;
                   if (TempElement.Masks.Count>0) and
                      (TempElement.Actions.Count>0) then
                    inherited Add(TempElement)
                   else
                    TempElement.Free;
                   TempElement:=tMAKFidoSearcherElement.Create;
                   continue;
               end;
              end;
mfsearAction: MaskFlag:=false;
       end;
   end;

      inc(i);
  end;

     if (TempElement.Masks.Count>0) and (TempElement.Actions.Count>0) then
      inherited Add(TempElement)
     else
      TempElement.Free;

     Index:=i;
end;

procedure tMAKFidoSearcherList.ClearElement(Index: integer);
 var
    TempElement :tMAKFidoSearcherElement;
begin
     if Assigned(inherited Items[Index]) then
 begin
      TempElement:=tMAKFidoSearcherElement(inherited Items[Index]);
      TempElement.Free;
      inherited Items[Index]:=nil;
  end;
end;

constructor tMAKFidoSearcherList.Create(const AArea: string);
begin
     inherited Create;
     FArea:=UpperCase(AArea);
end;

function tMAKFidoSearcherList.GetItem(
  Index: integer): tMAKFidoSearcherElement;
begin
     Result:=tMAKFidoSearcherElement(inherited Items[Index]);
end;

function tMAKFidoSearcherList.ProcessPDB(const Packet: tPalmFidoDB;
   var Index :integer) :boolean;
 var
    i          :integer;
    TempHeader :tSPacket;
    TempResult :tMFSActionCommand;
begin
     TempHeader:=Packet.Items[Index];

     Result:=UpperCase(StrPas(TempHeader.Area))={UpperCase(}Area{)};
     if (not Result) and (Area<>MFSAllAreasID) then
      exit;

     for i:=0 to Count-1 do
 begin
      TempResult:=Items[i].ProcessPDB(Packet,Index);
      if TempResult in [mfsaMove,mfsaDelete] then
  begin
       Result:=true;
       if TempResult=mfsaDelete then
        dec(Index);
       break;
   end;    
  end;
end;

{------------------------------------------------------------------------------}
{ tMAKFidoSearcher }

procedure tMAKFidoSearcher.Add(const Strings: tStrings);
 var
    i,n         :integer;
    TempElement :tMAKFidoSearcherList;
    s           :string;
begin
     i:=0;
     while i<Strings.Count do
 begin
      if CompareCommandWithColon(Strings[i],MFSCommandsStr[MFSCommandID_Area]) then
  begin
       n:=length(MFSCommandsStr[MFSCommandID_Area])+1;
       s:=trim(copy(Strings[i],n+1,length(Strings[i])-n));
       n:=IndexOfArea(s);
       if n=-1 then
        TempElement:=tMAKFidoSearcherList.Create(s)
       else
        TempElement:=Items[n];
       inc(i);
       TempElement.Add(Strings,i);
       if (n=-1) and (TempElement.Count>0) then
        inherited Add(TempElement);
   end
      else
       inc(i);
  end;
end;

procedure tMAKFidoSearcher.ClearElement(Index: integer);
 var
    TempElement :tMAKFidoSearcherList;
begin
     if Assigned(inherited Items[Index]) then
 begin
      TempElement:=tMAKFidoSearcherList(inherited Items[Index]);
      TempElement.Free;
      inherited Items[Index]:=nil;
  end;
end;

function tMAKFidoSearcher.GetItem(Index: integer): tMAKFidoSearcherList;
begin
     Result:=tMAKFidoSearcherList(inherited Items[Index]);
end;

function tMAKFidoSearcher.IndexOfArea(const AArea: string): integer;
 var
    i :integer;
begin
     for i:=0 to Count-1 do
      if {UpperCase(}Items[i].Area{)}=UpperCase(AArea) then
  begin
       Result:=i;
       exit;
   end;
     Result:=-1;
end;

procedure tMAKFidoSearcher.ProcessPDB(const Packet: tPalmFidoDB);
 var
    i,j,k,n        :integer;
begin
     i:=0;
     n:=Packet.Count;
     while i<n do
 begin
      k:=i;
      for j:=0 to Count-1 do
       if Items[j].ProcessPDB(Packet,i) then
        break;
      inc(i);
      if k=i then
       dec(n);
  end;
end;

end.
