unit MAKPDBLib;

{******************************************************************************}
{*                             MAKPalmToss                                    *}
{*                                                                            *}
{* (C) Copyright 2003 Dmitry Malenok                                          *}
{*                                                                            *}
{* FIDO mail tosser for FIDOReader form palm created by                       *}
{*                                      Sergey Konuhov (2:5022/59.14@fidonet) *}
{* Based on sources of Palm_Tosser created by Sergey Konuhov                  *}
{*                                                                            *}
{* It was creater for following compilers:                                    *}
{*   Windows 9x/NT/2K/XP   -  Delphi 6                                        *}
{*   OS/2 Warp 3/4/4.5     -  Free Pascal 1.0.7 (build 2003/03/01 or later)   *}
{*                                                                            *}
{*                                                                            *}
{******************************************************************************}

{$H+}

interface

 uses
{$IFDEF OS2}
     OS2Def,
{$ELSE}
     DateUtils,
{$ENDIF}
     Classes, SysUtils, PTTypes;

 const
      dmDBNameLength                  =32;
      TIME_DELTA                      =0;
      PDB_MAX_RECORD_NUM              =$FFFF;
      PDB_MAX_RECORD_LEN              =$FFFF;
      PDB_UNIQUE_ID                   =$4B5300;
      PDB_RECORD_ATTR                 =$40;


 Type
{*************************************************************}
{* Structure of a Record entry                               *}
{*************************************************************}
     tRecordEntry          =packed record
                             LocalChunkID :LocalID;               // local chunkID of a record
                             Attributes   :UInt8;                 // record attributes;
                             UniqueID     :array [0..2] of UInt8; // unique ID of record; should not be 0 for a legal record.
                            end;

{*************************************************************}
{* Structure of a Resource entry                             *}
{*************************************************************}
     tRsrcEntry            =packed record
                             RType         :UInt32;               // resource type
                             ID            :UInt16;               // resource id
                             LocalChunkID  :LocalID;              // resource data chunk ID
                            end;

{*********************************************************************}
{* Structure of a record list extension. This is used if all         *}
{* the database record/resource entries of a database cant fit into *}
{* the database header.                                              *}
{*********************************************************************}
     tRecordList           =packed record
                             NextRecordListID :LocalID;           // local chunkID of next list
                             NumRecords       :UInt16;            // number of records in this list
{                             FirstEntry       :UInt16;            // array of Record/Rsrc entries starts here}
                            end;

{*************************************************************}
{* Structure of a Database Header                            *}
{*************************************************************}
     tDatabaseHdr          =packed record
                             Name               :array [0..dmDBNameLength-1] of char;  // name of database
                             Attributes         :UInt16;                               // database attributes
                             Version            :UInt16;                               // version of database
                             CreationDate       :UInt32;                               // creation date of database
                             ModificationDate   :UInt32;                               // latest modification date
                             LastBackupDate     :UInt32;                               // latest backup date
                             ModificationNumber :UInt32;                               // modification number of database
                             AppInfoID          :LocalID;                              // application specific info
                             SortInfoID         :LocalID;                              // app specific sorting info
                             HType              :UInt32;                               // database type
                             Creator            :UInt32;                               // database creator
                             UniqueIDSeed       :UInt32;                               // used to generate unique IDs.
                                                                                       //Note that only the low order
                                                                                       //3 bytes of this is used (in
                                                                                       //RecordEntryType.uniqueID).
                                                                                       //We are keeping 4 bytes for
                                                                                       //alignment purposes.
                             RecordList         :tRecordList;                          // first record list
                            end;

     tpdbCreatorString     =string[4];
     tpdbTypeString        =string[4];

     tpdbRecord            =record
                             Size  :LongInt;
                             Buf   :pointer;
                            end;
     ppdbRecord            =^tpdbRecord;

     tPalmDB               =class(tMAKList)
                             private
                              FHeader          :tDatabaseHdr;

                              function GetItem(Index: LongInt): tpdbRecord;
                              procedure SetItem(Index: LongInt; const Value: tpdbRecord);
                             protected
                              procedure ClearElement(Index :integer); override;
                             public
                              constructor Create;

                              function Add(const Value :tpdbRecord) :LongInt;

                              function SaveToStream(Stream :tStream; StartNumber :LongWord; Limit :LongWord) :LongInt; virtual;
                              procedure SaveToFile(const FileName :string); virtual;
                              procedure LoadFromStream(Stream :tStream); virtual;
                              procedure LoadFromFile(const FileName :string); virtual;

                              property Header :tDatabaseHdr read FHeader write FHeader;

                              property Items[Index :LongInt] :tpdbRecord read GetItem write SetItem; default;
                            end;

 function pdbRevert16(Value :UInt16) :UInt16;
 function pdbRevert32(Value :UInt32) :UInt32;

implementation

function pdbRevert16(Value: UInt16): UInt16;
begin
     Result:=((Value and $ff) shl 8)+((Value shr 8) and $ff);
end;

function pdbRevert32(Value: UInt32): UInt32;
begin
     Result:=(pdbRevert16(Value and $ffff) shl 16) +
             pdbRevert16(Value shr 16);
end;

{ tPalmDB }

function tPalmDB.Add(const Value: tpdbRecord): LongInt;
 var
    TempItem :ppdbRecord;
begin
     new(TempItem);
     TempItem^.Size:=Value.Size;
     TempItem^.Buf:=Value.Buf;
     Result:=inherited Add(TempItem);
end;

procedure tPalmDB.ClearElement(Index: integer);
 var 
    TempRecrod :ppdbRecord;
    p          :pointer;
begin
     if Assigned(inherited Items[Index]) then
 begin
      if Assigned(Items[Index].Buf) then
  begin
       p:=Items[Index].Buf;
       FreeMem(p,Items[Index].Size);
   end;
      TempRecrod:=ppdbRecord(inherited Items[Index]);
      FreeMem(TempRecrod,sizeof(tpdbRecord));
  end;
end;

constructor tPalmDB.Create;
begin
     inherited Create;

     FillChar(FHeader,sizeof(FHeader),0);
     FHeader.CreationDate:=pdbRevert32(DateTimeToUnix(Now)+TIME_DELTA);
     FHeader.ModificationDate:=pdbRevert32(DateTimeToUnix(Now)+TIME_DELTA);
     FHeader.Creator:=$4B414D;
end;

function tPalmDB.GetItem(Index: LongInt): tpdbRecord;
begin
     Result:=ppdbRecord(inherited Items[Index])^;
end;

procedure tPalmDB.LoadFromFile(const FileName: string);
 var
    f :tFileStream;
begin
     f:=TFileStream.Create(FileName,fmOpenRead or fmShareDenyWrite);
     LoadFromStream(f);
     f.Free;
end;

procedure tPalmDB.LoadFromStream(Stream: tStream);
 Type
     tRecCountArray  =array [0..High(UInt16)] of UInt32;
     pRecCountArray  =^tRecCountArray;
 var
    Pos,StartPos   :UInt32;
    i              :LongInt;
    RecEntry       :tRecordEntry;
    RecCountArray  :pRecCountArray;
    n              :LongInt;
    TempRecord     :tpdbRecord;
begin
     StartPos:=Stream.Position;
     Stream.Read(FHeader,SizeOf(FHeader));

     n:=pdbRevert16(FHeader.RecordList.NumRecords);

     GetMem(RecCountArray,n*Sizeof(UInt32));
     try
      for i:=0 to n-1 do
  begin
       Stream.Read(RecEntry,sizeof(RecEntry));
       if i>0 then
        RecCountArray^[i-1]:=pdbRevert32(RecEntry.LocalChunkID)-Pos;
       Pos:=pdbRevert32(RecEntry.LocalChunkID);
   end;
      RecCountArray^[n-1]:=Stream.Size-StartPos-Pos;

      Stream.Position:=Stream.Position+Sizeof(UInt16);

      for i:=0 to n-1 do
  begin
       TempRecord.Size:=RecCountArray^[i];
       GetMem(TempRecord.Buf,TempRecord.Size);
       Stream.Read(TempRecord.Buf^,TempRecord.Size);
       Add(TempRecord);
   end;

      FreeMem(RecCountArray,n*Sizeof(UInt32));
     except
      FreeMem(RecCountArray,n*Sizeof(UInt32));
      Clear;
      raise;
     end;
end;

procedure tPalmDB.SaveToFile(const FileName: string);
 var
    f :tFileStream;
begin
     f:=TFileStream.Create(FileName,fmCreate or fmShareDenyWrite);
     SaveToStream(f,0,0);
     f.Free;
end;

function tPalmDB.SaveToStream(Stream: tStream; StartNumber :LongWord; Limit :LongWord) :LongInt;
 var
    Pos       :UInt32;
    i         :LongWord;
    RecEntry  :tRecordEntry;
    w         :word;
    n         :LongWord;
    TempSize  :LongWord;
begin
     TempSize:=SizeOf(FHeader)+sizeof(w);
     n:=Count-StartNumber;
     if Limit>0 then
      for i:=StartNumber to Count-1 do
  begin
       TempSize:=TempSize+Items[i].Size+Sizeof(tRecordEntry);
       if TempSize>Limit then
   begin
        n:=i-StartNumber;
        if n=0 then
         inc(n);
        break;
    end;
   end;

     FHeader.RecordList.NumRecords:=pdbRevert16(n);

     Stream.Write(FHeader,SizeOf(FHeader));

     Pos:=SizeOf(FHeader)+n*sizeof(tRecordEntry)+sizeof(w);

     RecEntry.Attributes:=PDB_RECORD_ATTR;
     for i:=StartNumber to StartNumber+n-1 do
 begin
      RecEntry.LocalChunkID:=pdbRevert32(Pos);
      RecEntry.UniqueID[0]:=((PDB_UNIQUE_ID+i) and $FF0000) shr 16;
      RecEntry.UniqueID[1]:=((PDB_UNIQUE_ID+i) and $FF00) shr 8;
      RecEntry.UniqueID[2]:=((PDB_UNIQUE_ID+i) and $FF);
      Stream.Write(RecEntry,sizeof(RecEntry));
      Inc(Pos,Items[i].Size);
  end;
      w:=0;
      Stream.Write(w,sizeof(w));

      for i:=StartNumber to StartNumber+n-1 do
       Stream.Write(Items[i].Buf^,Items[i].Size);

     Result:=StartNumber+n;
end;

procedure tPalmDB.SetItem(Index: LongInt; const Value: tpdbRecord);
 var
    TempRecord :ppdbRecord;
begin
     TempRecord:=ppdbRecord(inherited Items[Index]);
     if Assigned(TempRecord^.Buf) then
      FreeMem(TempRecord^.Buf,TempRecord^.Size);
     TempRecord^.Size:=Value.Size;
     TempRecord^.Buf:=Value.Buf;
end;

end.
