{******************************************************************************}
{*                             MAKPalmToss                                    *}
{*                                                                            *}
{* (C) Copyright 2003 Dmitry Malenok                                          *}
{*                                                                            *}
{* FIDO mail tosser for FIDOReader form palm created by                       *}
{*                                      Sergey Konuhov (2:5022/59.14@fidonet) *}
{* Based on sources of Palm_Tosser created by Sergey Konuhov                  *}
{*                                                                            *}
{* It was creater for following compilers:                                    *}
{*   Windows 9x/NT/2K/XP   -  Delphi 6                                        *}
{*   OS/2 Warp 3/4/4.5     -  Free Pascal 1.0.7 (build 2003/03/01 or later)   *}
{*                                                                            *}
{*                                                                            *}
{******************************************************************************}

var
   Config_Ini           :string;
   Pkt_Inbound          :string;
   Pkt_PDB              :string;

   AreaName             :TStringList;
   OutputFilters        :tMAKFidoSearcher;
   ConvertFilterStrings :boolean;
   LoadAreasFlag        :boolean;

   pkt_deleted          :boolean;
   pkt_count,
   ipkt_count           :LongInt;


procedure LoadIniSections(const IniFile :string);
 var
    fl          :tIniLikeStream;
    TempStrings :tStringList;
    i           :integer;
begin
     try
      fl:=tIniLikeStream.Create(IniFile,fmOpenRead+fmShareDenyWrite);
     except
      exit;
     end;

     try
      try
       if LoadAreasFlag then
        fl.ReadSectionFull('area_list',AreaName);
      except
       AreaName.Clear;
      end;
      TempStrings:=tStringList.Create;
      try
       fl.ReadSectionFull('output_filters',TempStrings);
       if ConvertFilterStrings then
        for i:=0 to TempStrings.Count-1 do
         TempStrings[i]:=OEMToAnsiStr(TempStrings[i]);
       OutputFilters.Add(TempStrings);
      except
       OutputFilters.Clear;
      end;
      TempStrings.Free;
     finally
      fl.Free;
     end;
end;


 var
    i                    :LongInt;
    pdb                  :tPalmFidoDB;
    fido                 :tMAKFIDOPacket;
    sr                   :TSearchRec;
{$IFDEF OS2}
 {$IFDEF FPC}
    IniFile              :tIniFile;
 {$ELSE}
    IniFile              :tAsciiIniFile;       //Ascii ini-file
 {$ENDIF}
{$ELSE}
    IniFile              :tIniFile;
{$ENDIF}
    LogFile              :tLogStream;
    s                    :string;
    ErrFlag              :integer;
    LimitSize            :LongWord;
    AppendFlag           :boolean;
    DupChecker           :tMAKFidoPDBMsgIDCheckFroDuplicates;
    DupCheckerFileName   :string;
    DupCheckerFileSize   :integer;
    DupCheckerAction     :tMCFDActionRec;
    TempFileTime         :LongInt;
    TempFileName         :string;
begin
     WriteLn(ProgramName+' '+VersionString+' '+CopyrightString);

     ErrFlag:=0;
     Config_Ini:=ExtractFilePath(ParamStr(0))+'toss_cfg.ini';

     AreaName:=TStringList.Create;
     OutputFilters:=tMAKFidoSearcher.Create;
     DupChecker:=nil;

     try
{$IFDEF OS2}
 {$IFDEF FPC}
      IniFile:=TIniFile.Create(Config_Ini);
 {$ELSE}
      IniFile:=TAsciiIniFile.Create(Config_Ini);
 {$ENDIF}
{$ELSE}
      IniFile:=TIniFile.Create(Config_Ini);
{$ENDIF}
     except
      Writeln(Format('Error. File: %s is not found in current directiry',[config_ini]));
      Halt(28);
     end;

     try
      Pkt_Inbound:=IniFile.ReadString('Directories','Inbound','');
      Pkt_PDB:=IniFile.ReadString('Directories','Tossed_PDB','');

      if Pkt_Inbound='' then
  begin
       Writeln(Format('Error. Can`t read "inbound" parameter from %s.',[config_ini]));
       Halt(28);
   end;

      if Pkt_PDB='' then
  begin
       Writeln(Format('Error. Can`t read "tossed_pdb" parameter from %s.',[config_ini]));
       Halt(28);
   end;

{$IFDEF OS2}
      if Pkt_Inbound[1]='"' then
       Pkt_Inbound:=copy(Pkt_Inbound,2,length(Pkt_Inbound)-1);
      if Pkt_Inbound[length(Pkt_Inbound)]='"' then
       Pkt_Inbound:=copy(Pkt_Inbound,1,length(Pkt_Inbound)-1);
      if not FileExists(Pkt_Inbound) then
{$ELSE}
      if not DirectoryExists(Pkt_Inbound) then
{$ENDIF}
  begin
       Writeln('The "inbound" dir is absent.');
       Halt(28);
   end;

{$IFDEF OS2}
      if Pkt_PDB[1]='"' then
       Pkt_PDB:=copy(Pkt_PDB,2,length(Pkt_PDB)-1);
      if Pkt_PDB[length(Pkt_PDB)]='"' then
       Pkt_PDB:=copy(Pkt_PDB,1,length(Pkt_PDB)-1);
      if not FileExists(Pkt_PDB) then
{$ELSE}
      if not DirectoryExists(Pkt_PDB) then
{$ENDIF}
  begin
       Writeln('The "tossed_pdb" dir is absent.');
       Halt(28);
   end;

      LoadAreasFlag:=IniFile.ReadInteger('Options','auto_fill_area_list',1)=0;

      pkt_deleted:=IniFile.ReadInteger('Options','delete_pkt_after_toss',0)=1;

      LimitSize:=IniFile.ReadInteger('Options','pkt_size_limit',0);
      AppendFlag:=IniFile.ReadInteger('Options','append_to_last_file',0)=1;
      ConvertFilterStrings:=IniFile.ReadInteger('Options','ConvertFilterStringsTo1251',0)=1;
      DupCheckerFileName:=IniFile.ReadString('Options','DupCheckerFileName','');
      DupCheckerFileSize:=IniFile.ReadInteger('Options','DupCheckerFileSize',0);
      DupCheckerAction.Destination:=IniFile.ReadString('Options','DupCheckerMoveTo','');
      s:=LowerCase(IniFile.ReadString('Options','DupCheckerAction',''));
      if s='move' then
  begin
       DupCheckerAction.Action:=mcfdMove;
       if length(DupCheckerAction.Destination)=0 then
        DupCheckerAction.Destination:='_Duplicates_';
   end
      else
       DupCheckerAction.Action:=mcfdDelete;

      s:=IniFile.ReadString('Options','log_file_name','');
     finally
      IniFile.Free;
     end;

     try
      LoadIniSections(Config_Ini);

      if FileExists(s) then
       LogFile:=tLogStream.Create(s,fmOpenReadWrite+fmShareDenyWrite)
      else
       LogFile:=tLogStream.Create(s,fmCreate);
      LogFile.Position:=LogFile.Size;
     except
      Writeln('Error. Can`t read "log_file_name" parameter from ini file or create the file. Loging is disabled.');
      LogFile:=nil;
     end;

     pkt_count:=0;
     ipkt_count:=0;

     if Assigned(LogFile) then
      LogFile.AddToLogLn(FormatDateTime('dd mmm yyyy',Now)+', '+ProgramName+' '+VersionString+' '+CopyrightString);

     try
      if (DupCheckerFileName<>'') and (DupCheckerFileSize>0) then
       DupChecker:=tMAKFidoPDBMsgIDCheckFroDuplicates.Create(DupCheckerFileSize);
       if FileExists(DupCheckerFileName) then
        try
         if Assigned(LogFile) then
          LogFile.AddToLog('Loading Duplicates Database...');
         DupChecker.LoadFromFile(DupCheckerFileName);
         if Assigned(LogFile) then
          LogFile.WriteLnString('Ok');
        except
         if Assigned(LogFile) then
          LogFile.WriteLnString('Fails');
         DupChecker.Free;
         DupChecker:=nil;
        end;
     except
      DupChecker:=nil;
     end;

     if Pkt_Inbound[length(Pkt_Inbound)]<>'\' then
      Pkt_Inbound:=Pkt_Inbound+'\';
     if Pkt_PDB[length(Pkt_PDB)]<>'\' then
      Pkt_PDB:=Pkt_PDB+'\';

     try

//Tossing

{$IFDEF OS2}
      if FindFirst(Pkt_Inbound+'*.pkt',faAnyFile and not faDirectory,sr)=0 then
{$ELSE}
      if FindFirst(Pkt_Inbound+'*.pkt',faAnyFile and not faVolumeID and not faDirectory,sr)=0 then
{$ENDIF}
  begin
       pdb:=tPalmFidoDB.Create;
       fido:=tMAKFidoPacket.Create;

       repeat
        write(Format('Tossing packet: %s - ',[sr.Name]));

        try
         fido.Free;
         fido:=tMAKFidoPacket.Create;

         fido.LoadFromFile(Pkt_Inbound+sr.Name);

         Inc(ipkt_count);

         for i:=0 to fido.Packets.Count-1 do
     begin
          if AreaName.Count>0 then
      begin
           if AreaName.IndexOf(fido.Packets[i].Area)<>-1 then
       begin
            pdb.Add(fido.Packets[i]);
            Inc(pkt_count);
        end
       end
          else
      begin
           pdb.Add(fido.Packets[i]);
           Inc(pkt_count);
       end;
      end;
         if Assigned(LogFile) then
          LogFile.AddToLogLn(Format('Processing inbound packet %s - Ok (%4d message(s) is read)',[sr.Name,fido.Packets.Count]));
         if pkt_deleted then
     begin
          if Assigned(LogFile) then
           LogFile.AddToLogLn(Format('Delete inbound packet %s',[sr.Name]));
          if DeleteFile(Pkt_Inbound+sr.Name) then
           write('Read & deleted.')
          else
           write('Read & not deleted.');
      end
         else
          write('Ok.');
         writeln(Format(' (%4d message(s) is read)',[fido.Packets.Count]));
        except
         if Assigned(LogFile) then
          LogFile.AddToLogLn(Format('Processing inbound packet %s - Error',[sr.Name]));
         writeln('Error.');
        end;

       until FindNext(sr)<>0;

       FindClose(sr);

       s:=Format('Inbound packets: %d, messages: %d',[ipkt_count,pkt_count]);
       writeln(s);
       if Assigned(LogFile) then
        LogFile.AddToLogLn(s);


       if Assigned(DupChecker) then
   begin
        s:='Check for duplicates... ';
        write(s);
        if Assigned(LogFile) then
         LogFile.AddToLog(s);

        s:=Format('%d duplicates is found',[DupChecker.CheckFidoPDB(pdb,DupCheckerAction)]);
        writeln(s);
        if Assigned(LogFile) then
         LogFile.WriteLnString(s);

        try
         DupChecker.SaveToFile(DupCheckerFileName);
        except
         s:=Format('Can'#27't write duplicates database into %s.',[DupCheckerFileName]);
         writeln(s);
         if Assigned(LogFile) then
          LogFile.AddToLogLn(s);
        end;
    end;

       if pkt_count<>0 then
   begin
        s:='Filtering...';
        writeln(s);
        if Assigned(LogFile) then
         LogFile.AddToLogLn(s);

        OutputFilters.ProcessPDB(pdb);
    end;     

       if pkt_count<>0 then
   begin
        TempFileName:='';
        if AppendFlag then
{$IFDEF OS2}
         if FindFirst(Pkt_PDB+'*.pdb',faAnyFile and not faDirectory,sr)=0 then
{$ELSE}
         if FindFirst(Pkt_PDB+'*.pdb',faAnyFile and not faVolumeID and not faDirectory,sr)=0 then
{$ENDIF}
     begin
          TempFileName:=sr.Name;
          i:=sr.Size;
          TempFileTime:=sr.Time;
          while FindNext(sr)=0 do
           if (FileDateToDateTime(TempFileTime)<FileDateToDateTime(sr.Time)) or
              ((FileDateToDateTime(TempFileTime)=FileDateToDateTime(sr.Time)) and
               (i>sr.Size)) then
       begin
            TempFileName:=sr.Name;
            i:=sr.Size;
            TempFileTime:=sr.Time;
        end;

          FindClose(sr);

          i:=pdb.Count;
          pdb.LoadFromFile(Pkt_PDB+TempFileName);
          if Assigned(LogFile) then
           LogFile.AddToLogLn(Format('%d pakets is read from %s',[pdb.Count-i,TempFileName]));
          if not RenameFile(Pkt_PDB+TempFileName,Pkt_PDB+TempFileName+'.tmp') then
           raise Exception.Create('Can'#$27't rename '+Pkt_PDB+TempFileName);
      end;

        pdb.SortAreas;
        s:=pdb.SaveToDir(Pkt_PDB,TempFileName,LimitSize);

        if length(TempFileName)>0 then
         DeleteFile(Pkt_PDB+TempFileName+'.tmp');
        s:=Format('Created packets: %s',[s]);
        Writeln(s);
        if Assigned(LogFile) then
         LogFile.AddToLogLn(s);
    end;
   end;

     writeln('Done.');
     except
      writeln('Run-time error.');
      if Assigned(LogFile) then
       LogFile.AddToLogLn('Run-time error.');
      ErrFlag:=1;
     end;
      if Assigned(LogFile) then
       LogFile.WriteLnString('');

      LogFile.Free;
      OutputFilters.Free;
      AreaName.Free;
      Halt(ErrFlag);
end.

