unit MAKSearcherUnit;

{******************************************************************************}
{*                             MAKPalmToss                                    *}
{*                                                                            *}
{* (C) Copyright 2003 Dmitry Malenok                                          *}
{*                                                                            *}
{* FIDO mail tosser for FIDOReader form palm created by                       *}
{*                                      Sergey Konuhov (2:5022/59.14@fidonet) *}
{* Based on sources of Palm_Tosser created by Sergey Konuhov                  *}
{*                                                                            *}
{* It was creater for following compilers:                                    *}
{*   Windows 9x/NT/2K/XP   -  Delphi 6                                        *}
{*   OS/2 Warp 3/4/4.5     -  Free Pascal 1.0.7 (build 2003/03/01 or later)   *}
{*                                                                            *}
{*                                                                            *}
{******************************************************************************}

{$H+}

interface

 uses
     Classes;

 type
     tMAKStringETypes        =(msetStrings,msetAny,msetAnyOne,msetAnyFigure,msetNotInStrings);
     tMAKStringSplitElement  =record
                               EType    :tMAKStringETypes;
                               Strings  :tStringList;
                              end;
     pMAKStringSplitElement  =^tMAKStringSplitElement;

     tMAKStringSplitArray    =Class(tList)
                               private
                                function GetItem(Index: integer): tMAKStringSplitElement;
                                procedure SetItem(Index: integer; const Value: tMAKStringSplitElement);
                               protected
                                function Add(const Element :tMAKStringSplitElement) :integer;
                               public
                                constructor Create(const s :string);

                                procedure Clear; override;

                                property Items[Index :integer] :tMAKStringSplitElement read GetItem write SetItem; default;
                              end;

     tMAKStringSearcher      =class
                               private
                                FSearchString   :string;

                               protected
                                FSplitString    :tMAKStringSplitArray;

                                function GetSearchString: string; virtual;
                                procedure SetSearchString(const Value: string); virtual;
                               public
                                constructor Create;
                                destructor Destroy; override;
                                
                                function SearchInString(const s :string) :boolean;

                                property SearchString :string read GetSearchString write SetSearchString;
                               end;

implementation

{ tMAKStringSplitArray }

function tMAKStringSplitArray.Add(
  const Element: tMAKStringSplitElement): integer;
 var
    TempElement :pMAKStringSplitElement;
begin
     new(TempElement);
     with TempElement^ do
 begin
      EType:=Element.EType;
      Strings:=Element.Strings;
  end;
     Result:=inherited Add(TempElement);
end;

procedure tMAKStringSplitArray.Clear;
 var
    i           :integer;
    TempElement :pMAKStringSplitElement;
begin
     for i:=0 to Count-1 do
      if Assigned(inherited Items[i]) then
  begin
       TempElement:=inherited Items[i];
       TempElement^.Strings.Free;
       dispose(TempElement);
       inherited Items[i]:=nil;
   end;

     inherited Clear;
end;

constructor tMAKStringSplitArray.Create(const s: string);
 const
      SpecialSymbols =['*','?','#',',','\','[',']'];

 function NormalizeString(const s :string) :string;
   var
      i,n :integer;
  begin
       Result:='';
       i:=1;
       n:=length(s);
       while i<=n do
   begin
        if (s[i]='\') and (i<n) then
    begin
         inc(i);
         if s[i] in SpecialSymbols then
          Result:=Result+s[i]
         else
      begin
           dec(i,1);
           Result:=Result+'\';
       end;
     end
        else
         Result:=Result+s[i];
        inc(i);
    end;
   end;
 var
    i,j,n,l      :integer;
    TempElement  :tMAKStringSplitElement;
    SlashFlag    :boolean;
    NotFlag      :boolean;
begin
     j:=1;
     i:=1;
     n:=length(s);
     while i<=n do
 begin
      case s[i] of
   '*': begin
             if i>j then
         begin
              TempElement.EType:=msetStrings;
              TempElement.Strings:=TStringList.Create;
              TempElement.Strings.Add(NormalizeString(copy(s,j,i-j)));
              Add(TempElement);
          end;
             if not ((Count>0) and (Items[Count-1].EType=msetAny)) then
         begin
              TempElement.EType:=msetAny;
              TempElement.Strings:=nil;
              Add(TempElement);
          end;    
             j:=i+1;
         end;
   '?': begin
             if i>j then
         begin
              TempElement.EType:=msetStrings;
              TempElement.Strings:=TStringList.Create;
              TempElement.Strings.Add(NormalizeString(copy(s,j,i-j)));
              Add(TempElement);
          end;
             if not ((Count>0) and (Items[Count-1].EType=msetAny)) then
         begin
              TempElement.EType:=msetAnyOne;
              TempElement.Strings:=nil;
              Add(TempElement);
          end;    
             j:=i+1;
         end;
   '#': begin
             if i>j then
         begin
              TempElement.EType:=msetStrings;
              TempElement.Strings:=TStringList.Create;
              TempElement.Strings.Add(NormalizeString(copy(s,j,i-j)));
              Add(TempElement);
          end;
             TempElement.EType:=msetAnyFigure;
             TempElement.Strings:=nil;
             Add(TempElement);
             j:=i+1;
         end;
   '\': begin
             if (i<n) and (s[i+1] in SpecialSymbols) then
              inc(i);
         end;
   '[': begin
             l:=j;
             j:=i+1;
             SlashFlag:=false;

             NotFlag:=(j<n) and (s[j]='!');
             if NotFlag then
              inc(j);

             while j<=n do
         begin
              case s[j] of
           '\': SlashFlag:=not SlashFlag;
           ']': if SlashFlag then
                 SlashFlag:=false
                else
                 break;
               else
                SlashFlag:=false;
               end;
              inc(j);
          end;

             if j<=n then
         begin
              if i>l then
          begin
               TempElement.EType:=msetStrings;
               TempElement.Strings:=TStringList.Create;
               TempElement.Strings.Add(NormalizeString(copy(s,l,i-l)));
               Add(TempElement);
           end;

              if NotFlag then
          begin
               TempElement.EType:=msetNotInStrings;
               inc(i);
           end
              else
               TempElement.EType:=msetStrings;
              TempElement.Strings:=TStringList.Create;

              inc(i);
              l:=i;
              while l<j do
          begin
               case s[l] of
            '\': SlashFlag:=not SlashFlag;
            ',': if SlashFlag then
                  SlashFlag:=false
                 else
             begin
                  TempElement.Strings.Add(NormalizeString(copy(s,i,l-i)));
                  i:=l+1;
              end;
                else
                 SlashFlag:=false;
               end;

               inc(l);
           end;

              if i<l then
               TempElement.Strings.Add(NormalizeString(copy(s,i,l-i)));

              i:=j;
              inc(j);

              Add(TempElement);
          end;
         end;
      end;

      inc(i);
  end;

     if i>j then
 begin
      TempElement.EType:=msetStrings;
      TempElement.Strings:=TStringList.Create;
      TempElement.Strings.Add(NormalizeString(copy(s,j,i-j)));
      Add(TempElement);
  end;
end;

function tMAKStringSplitArray.GetItem(
  Index: integer): tMAKStringSplitElement;
begin
     Result:=pMAKStringSplitElement(inherited Items[Index])^;
end;

procedure tMAKStringSplitArray.SetItem(Index: integer;
  const Value: tMAKStringSplitElement);
 var
    TempElement :pMAKStringSplitElement;
begin
     TempElement:=inherited Items[Index];
     with TempElement^ do
 begin
      Strings.Free;
      EType:=Value.EType;
      Strings:=Value.Strings;
  end;
end;

{------------------------------------------------------------------------------}
{ tMAKStringSearcher }

constructor tMAKStringSearcher.Create;
begin
     inherited Create;

     FSplitString:=nil;
end;

destructor tMAKStringSearcher.Destroy;
begin
     if Assigned(FSplitString) then
      FSplitString.Free;
     inherited Destroy;
end;

function tMAKStringSearcher.GetSearchString: string;
begin
     Result:=FSearchString;
end;

function tMAKStringSearcher.SearchInString(const s: string): boolean;
 function SearchInStr(StartIndex, StartChar :integer) :boolean;
  var
     i,j,l     :integer;
     s1        :string;
  begin
       Result:=true;
       i:=StartIndex;
       j:=StartChar;

       while i<FSplitString.Count do
   begin
        case FSplitString[i].EType of
     msetStrings: begin
                       s1:=copy(s,j,length(s)-j+1);
                       for l:=0 to FSplitString[i].Strings.Count-1 do
                        if Pos(FSplitString[i].Strings[l],s1)=1 then
                    begin
                         if SearchInStr(i+1,j+length(FSplitString[i].Strings[l])) then
                          exit;

                         Result:=false;
                         exit;
                     end;
                       Result:=false;
                       exit;
                   end;
         msetAny: begin
                       if i+1=FSplitString.Count then
                        exit;
                       while j<=length(s) do
                   begin
                        if SearchInStr(i+1,j) then
                         exit;
                        inc(j);
                    end;
                       Result:=false;
                       exit;
                   end;
      msetAnyOne: begin
                       if j>length(s) then
                   begin
                        Result:=false;
                        exit;
                    end;
                       inc(j);
                   end;
   msetAnyFigure: begin
                       if j>length(s) then
                   begin
                        Result:=false;
                        exit;
                    end;
                       if not (s[j] in ['0'..'9']) then
                   begin
                        Result:=false;
                        exit;
                    end;
                       inc(j);
                   end;
msetNotInStrings: begin
                       s1:=copy(s,j,length(s)-j+1);
                       for l:=0 to FSplitString[i].Strings.Count-1 do
                        if Pos(FSplitString[i].Strings[l],s1)=1 then
                    begin
                         Result:=false;
                         exit;
                     end;
                   end;
        end;

        inc(i);
    end;

       Result:=j>length(s);

   end;
begin
     Result:=SearchInStr(0,1);
end;

procedure tMAKStringSearcher.SetSearchString(const Value: string);
begin
     FSearchString:=Value;
     if Assigned(FSplitString) then
      FSplitString.Free;
     FSplitString:=tMAKStringSplitArray.Create(Value); 
end;



end.
