unit PTTypes;

{******************************************************************************}
{*                             MAKPalmToss                                    *}
{*                                                                            *}
{* (C) Copyright 2003 Dmitry Malenok                                          *}
{*                                                                            *}
{* FIDO mail tosser for FIDOReader form palm created by                       *}
{*                                      Sergey Konuhov (2:5022/59.14@fidonet) *}
{* Based on sources of Palm_Tosser created by Sergey Konuhov                  *}
{*                                                                            *}
{* It was creater for following compilers:                                    *}
{*   Windows 9x/NT/2K/XP   -  Delphi 6                                        *}
{*   OS/2 Warp 3/4/4.5     -  Free Pascal 1.0.7 (build 2003/03/01 or later)   *}
{*                                                                            *}
{*                                                                            *}
{******************************************************************************}

{$H+}

interface

 uses
{$IFDEF OS2}
     OS2Def,
{$ELSE}
     Windows,
{$ENDIF}
     Classes, SysUtils;

{$IFDEF FPC}
{$IFDEF LANG_GERMAN}
{$i constsg.inc}
{$ELSE}
{$IFDEF LANG_SPANISH}
{$i constss.inc}
{$ELSE}
{$i constse.inc}
{$ENDIF}
{$ENDIF}
{$ENDIF}

 const
{$IFDEF OS2}
      ProgramName       ='MAKPalmToss/2';
{$ELSE}
      ProgramName       ='MAKPalmToss/W32';
{$ENDIF}
      VersionString     ='v1.0.1.4';
      CopyrightString   ='(C) Copyright 2003 Dmitry Malenok.';

      EngMonthNames :array[1..12] of string[3]=('Jan','Feb','Mar','Apr','May',
                                     'Jun','Jul','Aug','Sep','Oct','Nov','Dec');

{$IFDEF OS2}
{$IFNDEF FPC}
      SReadError        ='Stream read error';
      SWriteError       ='Stream write error';
{$ENDIF}
{$ENDIF}


 type
     UInt8              =Byte;
{IFDEF DELPHI}
     UShort             =Word;
     ULong              =LongWord;
{ENDIF}
     UInt16             =UShort;
     UInt32             =Ulong;
     LocalID            =Ulong;

     pUint8             =^UInt8;

     tTextStream        =class(tFileStream)
                          public
                           function ReadString :string;

                           procedure WriteString(const s :string);
                           procedure WriteLnString(const s :string);

                           function EOF :boolean;
                         end;

     tLogStream         =class(tTextStream)
                          public
                           procedure AddToLog(const s :string);
                           procedure AddToLogLn(const s :string);
                         end;

     tIniLikeStream     =class(tTextStream)
                          public
                           procedure ReadSectionFull(const Section :string;
                                                     Strings :tStrings);
                         end;

     tMAKList           =class(tList)
                          protected
                           procedure ClearElement(Index :integer); virtual; abstract;
                          public
                           procedure Clear; override;
                           procedure Delete(Index :integer);
                         end;

{$IFDEF OS2}
function DateTimeToUnix(const AValue: TDateTime): Longint;
procedure AltToWin(Src,Dst :pChar; Size :LongInt);
{$ENDIF}
{$IFDEF FPC}
Function FileExists(Const FileName: String): Boolean;
{$ENDIF}
function AsciiDateToDateTime(const s: string): tDateTime;
function OEMToAnsiStr(const s: string): string;

implementation

{$IFDEF OS2}
 const
{$IFDEF FPC}
      UnixDateDelta = 25569;
{$ELSE}
      UnixDateDelta = 7.19163000000000E+0005;
{$ENDIF}

function DateTimeToUnix(const AValue: TDateTime): Longint;
begin
     Result := Round((AValue - UnixDateDelta) * SecsPerDay);
end;
{$ENDIF}

{$IFDEF FPC}
Function FileExists(Const FileName: String): Boolean;
Var
  SearchRec: TSearchRec;
Begin
  If FindFirst(FileName, faAnyFile, SearchRec) = 0 Then
  Begin
    FileExists := True;
    FindClose(SearchRec);
  End
  Else FileExists := False;
End;
{$ENDIF}

function AsciiDateToDateTime(const s: string): tDateTime;
 var
    i,j,k           :LongInt;
    d               :array [1..6] of word;
begin
     j:=1;

     for i:=0 to High(d) do
      d[i]:=0;

     i:=1;

     while i<=length(s) do
 begin
      if j=2 then
  begin
       for k:=1 to 12 do
        if Copy(s,i,3)=EngMonthNames[k] then
    begin
         d[j]:=k;
         break;
     end;
       i:=i+3;
   end;
      if s[i] in [' ',':'] then
  begin
       while (i<length(s)) and (s[i+1]=' ') do
        inc(i);
       inc(j);
       if j>High(d) then break;
   end
      else
       d[j]:=d[j]*10+ord(s[i])-$30;

      inc(i);
  end;

     if d[3]<70 then d[3]:=d[3]+2000
                else d[3]:=d[3]+1900;

     try
      Result:=EncodeDate(d[3],d[2],d[1])+EncodeTime(d[4],d[5],d[6],0);
     except
      Result:=0;
     end;
end;

{$IFDEF OS2}
procedure AltToWin(Src,Dst :pChar; Size :LongInt);
 const
      WIN  :string='---+---L+T+-+LT=+TTLL-++-----v';
 var
    i   :LongInt;
    c   :char;
begin
     for i:=0 to Size-1 do
 begin
      c:=Src[i];
      if c>#$7f then
       Dst[i]:=Win[ord(c)-$7f]
      else
       Dst[i]:=c;
  end;
end;
{$ENDIF}

function OEMToAnsiStr(const s: string): string;
begin
     SetLength(Result, Length(s));
     if Length(Result)>0 then
{$IFDEF OS2}
      AltToWin(pchar(@s[1]),pchar(@Result[1]),length(Result));
{$ELSE}
{$IFDEF WIN32}
      OemToCharBuff(PChar(s), PChar(Result), Length(Result));
{$ELSE}
      OemToAnsiBuff(@s[1], @Result[1], Length(Result));
{$ENDIF}
{$ENDIF}
end;

{ tTextStream }

function tTextStream.EOF: boolean;
begin
     Result:=Size<=Position;
end;

function tTextStream.ReadString: string;
 var
    c         :char;
    BreakFlag :boolean;
begin
     Result:='';

     BreakFlag:=false;

     while not EOF do
 begin
      Read(c,SizeOf(c));
      if c in [#$0d,#$0a] then
  begin
       BreakFlag:=true;
       continue;
   end;

      if BreakFlag then
  begin
       Position:=Position-1;
       break;
   end;

      Result:=Result+c;
  end;

end;

procedure tTextStream.WriteLnString(const s: string);
begin
     WriteString(s+#$0d#$0a);
end;

procedure tTextStream.WriteString(const s: string);
begin
     Write(s[1],length(s));
end;

{ tLogStream }

procedure tLogStream.AddToLog(const s: string);
begin
     WriteString(FormatDateTime('hh:mm:ss ',Now)+s);
end;

procedure tLogStream.AddToLogLn(const s :string);
begin
     AddToLog(s+#$0d#$0a);
end;

{ tIniLikeStream }
procedure tIniLikeStream.ReadSectionFull(const Section: string;
  Strings: tStrings);
 var
    s              :string;
    SectionStarted :boolean;
    n              :integer;
begin
     Position:=0;
     SectionStarted:=false;
     n:=length(Section)+2;

     while not EOF do
 begin
      s:=ReadString;
      if (length(s)>0) and (s[1]<>';') then
  begin
       if not SectionStarted then
   begin
        SectionStarted:=(s[1]='[') and
                        (length(s)=n) and
                        (LowerCase(copy(s,2,n-2))=LowerCase(Section));
     end
        else
    begin
         if s[1]='[' then break;
         Strings.Add(s);
     end;
    end;
   end;
end;

{ tMAKList }

procedure tMAKList.Clear;
 var
    i :integer;
begin
     for i:=0 to Count-1 do
      ClearElement(i);
     inherited Clear;
end;

procedure tMAKList.Delete(Index: integer);
begin
     ClearElement(Index);
     inherited Delete(Index);
end;

end.
